<?php
/*
 * bNamed plug for Odin 1.2
 */

include 'classBNamed.php';

function BNamed_getConfigArray() {
    $configArray = array(
        "Username" => array("type" => "text", "size" => "20", "description" => "Enter your UID"),
        "Password" => array("type" => "password", "size" => "20", "description" => "Enter your password"),
        "VanityNS" => array("type" => "text", "size" => "20", "description" => "Only change this if you use our NS with vanity names", "defaultValue" => "ns.nscluster."),
        "noAdditionalFieldsTLDs" => array("type" => "text", "size" => "200", "description" => "comma separated list of TLD's for which no additional fields are requested", "defaultValue" => "com,net,org,biz,info,be"),
        "autoTrustee" => array("type" => "text", "size" => "200", "description" => "comma separated list of TLD's for which trustee service may automatically be activated (attention: extra charges may apply)", "defaultValue" => "de"),
        "Proxy" => array("type" => "text", "size" => "40", "description" => "Empty if no proxy is needed. If you do need to use a proxy, enter it like tcp://10.0.0.10:3128", "defaultValue" => "")
    );
    return $configArray;
}

function BNamed_RegisterNameservers($params) {
// Registering nameservers occurs automagically in our backend
// If Odin would provide us with IP-addresses in this function, we might want to save them in "additionalfields" so that they can be used in the update or purchase command
// ... but Odin doesn't provide IP's here
}

function BNamed_GetNameserversStatus($params) {
// Registering nameservers occurs automagically in our backend
    return array("completed" => true);
}

function BNamed_SaveNameservers($params) {
    $return = array();

    if (!strrpos(" " . $params["ns1"], "ns.nscluster.") && !strrpos(" " . $params["ns1"], $params["VanityNS"])) {

        $bNamed = new BNamed($params);
        $bNamed->AddParam("command", "update");
        $bNamed->AddParamsToIdentifyDomain($params);
        $bNamed->AddParam("DNSList", $bNamed->ExtractNameservers($params));
        $bNamed->AddParam("dnstype", "E");

        $bNamed->SendRequest();

        if ($bNamed->ParsedResponse->ErrorCode > 0) {
            $return["error"] = $bNamed->ParsedResponse->ErrorText . " ( ErrorCode = " . $bNamed->ParsedResponse->ErrorCode . " )";
        }
    } else {
        $return["error"] = "Using our nameservers, nothing changed";
    }
    return $return;
}

function BNamed_SetDNSSEC($params) {
    $return = array();

    $bNamed = new BNamed($params);
    $bNamed->AddParam("command", "update");
    $bNamed->AddParamsToIdentifyDomain($params);
    $bNamed->AddParam("RemoveDNSSec", "all");

    if (array_key_exists("dnsSecInfo", $params)) {
        $keys = $params["dnsSecInfo"];
        $i = 0;
        foreach ($keys as $key) {
            $count = $i++;
            $bNamed->AddParam("AddDnsSecAlgorithm" . $count, $key["keyAlg"]);
            $bNamed->AddParam("AddDnsSecPublicKey" . $count, $key["key"]);
        }
    }

    $bNamed->SendRequest();

    if ($bNamed->ParsedResponse->ErrorCode > 0) {
        $return["error"] = $bNamed->ParsedResponse->ErrorText . " ( ErrorCode = " . $bNamed->ParsedResponse->ErrorCode . " )";
    }
    return $return;
}

function BNamed_SaveRegistrarLock($params) {
    $return = array();

    $bNamed = new BNamed($params);
    $bNamed->AddParam("command", "lock");
    $bNamed->AddParamsToIdentifyDomain($params);
    $bNamed->AddParam("noEmail", "1");

    if ($params["lockenabled"] == 'locked') {
        $bNamed->AddParam("lock", 1);
    } else {
        $bNamed->AddParam("lock", 0);
    }

    $bNamed->SendRequest();

    if ($bNamed->ParsedResponse->ErrorCode > 0) {
        $return["error"] = $bNamed->ParsedResponse->ErrorText . " ( ErrorCode = " . $bNamed->ParsedResponse->ErrorCode . " )";
    }

    return $return;
}

function BNamed_IDProtectToggle($params) {
    $bNamed = new BNamed($params);
    $bNamed->AddParam("command", "update");
    $bNamed->AddParamsToIdentifyDomain($params);

    if ($params['protectenable']) {
        $bNamed->AddParam("whoisprivacy", "1");
    } else {
        $bNamed->AddParam("whoisprivacy", "0");
    }

    $bNamed->SendRequest();

    $values = array();

    if ($bNamed->ParsedResponse->ErrorCode == 0) {
//
    } else {
        $values['error'] = $bNamed->ParsedResponse->ErrorText;
    }

    return $values;
}

function BNamed_RegisterDomain($params) {
    $bNamed = new BNamed($params);
    return $bNamed->purchase($params, "N");
}

function BNamed_Sync($params) {
    $bNamed = new BNamed($params);
    $bNamed->AddParam("command", "GetDomain");
    $bNamed->AddParamsToIdentifyDomain($params);
    
    $bNamed->SendRequest();

    $return = array();

    if ($bNamed->ParsedResponse->ErrorCode == 0) {
        $statusCode = $bNamed->ParsedResponse->Result->StatusCode;
        if ($statusCode == "A" || $statusCode == "ANS") {
            $return['completed'] = true;
            date_default_timezone_set("Europe/Brussels");
            $return['expirydate'] = date('Y-m-d', strtotime($bNamed->ParsedResponse->Result->Expiration));
        } elseif (($statusCode == "B2"  || $statusCode == "B3") && !strpos("," . $bNamed->instantTransfer . ",", "," . $params["tld"] . ",") === FALSE) {
            //If it's an instantTransfer TLD, then something went wrong if it's pending confirmation from licensee or waiting whois check
            $return['error'] = "Transfer should have been instant, but is still pending. Probably caused by faulty auth code.";
        } elseif ($statusCode == "T2"  || $statusCode == "B2"  || $statusCode == "B3"  || $statusCode == "B4"  || $statusCode == "B5"  || $statusCode == "N2") {
            $return['completed'] = false;
        } else {
            $return['error'] = $bNamed->ParsedResponse->Result->StatusText;
        }
    } elseif ($bNamed->ParsedResponse->ErrorCode == 121) { 
        $return['error'] = $bNamed->ParsedResponse->ErrorText;
    } else {
        $return['completed'] = false;
    }

    return $return;
}

function BNamed_TransferDomain($params) {
    $bNamed = new BNamed($params);
    
    /*
    For .be we want to be able to both request a "trade" or a "transfer" via 
    BNamed_TransferDomain. Since a "trade" actually becomes a transfer-request
    but without verifying if the domain name is already in your account, we always
    request a trade
    */
    if ($params["tld"] == "be") {
	    return $bNamed->purchase($params, "T");
	} else {
            return $bNamed->purchase($params, "B");
	}
}

function BNamed_TransferSync($params) {
    return BNamed_Sync($params);
}

function BNamed_RenewDomain($params) {
    $return = array();

    $bNamed = new BNamed($params);
    $bNamed->AddParam("command", "Extend");
    $bNamed->AddParamsToIdentifyDomain($params);
    $bNamed->AddParam("NumYears", $params["regperiod"]);

    $bNamed->SendRequest();

    if ($bNamed->ParsedResponse->ErrorCode > 0) {
        $return["error"] = $bNamed->ParsedResponse->ErrorText . " ( ErrorCode = " . $bNamed->ParsedResponse->ErrorCode . " )";
    } else {
        $return["additionalfields"] = array("expirationdate" => $bNamed->ParsedResponse->Result->Expiration);
    }

    return $return;
}

function BNamed_SaveContactDetails($params) {
    $return = array();

    $bNamed = new BNamed($params);
    $bNamed->AddParam("command", "update");
    $bNamed->AddParamsToIdentifyDomain($params);
    $bNamed->addContactParams($params);

    $bNamed->SendRequest();

    if ($bNamed->ParsedResponse->ErrorCode > 0) {
        $return["error"] = $bNamed->ParsedResponse->ErrorText . " ( ErrorCode = " . $bNamed->ParsedResponse->ErrorCode . " )";
    } else {
        $return["completed"] = true;
    }

    return $return;
}

function BNamed_RequestDelete($params) {
    $return = array();

    $bNamed = new BNamed($params);
    $bNamed->AddParam("command", "ChangeExtendType");
    $bNamed->AddParam("ExtendType", "-1");
    $bNamed->AddParamsToIdentifyDomain($params);

    $bNamed->SendRequest();

    //ErrorCode 122 is returned if a domain name is not found. We assume the delete is no longer needed of the domain name is not there.
    if ($bNamed->ParsedResponse->ErrorCode > 0 and $bNamed->ParsedResponse->ErrorCode != 122) {
        $return["error"] = $bNamed->ParsedResponse->ErrorText . " ( ErrorCode = " . $bNamed->ParsedResponse->ErrorCode . " )";
    } else {
        //Success
    }

    return $return;
}

function BNamed_RequestDeleteSync($params) {
    $return = array();

    $bNamed = new BNamed($params);
    $bNamed->AddParam("command", "GetDomain");
    $bNamed->AddParamsToIdentifyDomain($params);

    $bNamed->SendRequest();

    //ErrorCode 122 is returned if a domain name is not found, thus we assume the delete is completed
    if ($bNamed->ParsedResponse->ErrorCode == 121) {
        $return["completed"] = true;
    } elseif ($bNamed->ParsedResponse->ErrorCode > 0) {
        $return["error"] = $bNamed->ParsedResponse->ErrorText . " ( ErrorCode = " . $bNamed->ParsedResponse->ErrorCode . " )";
    } elseif ($bNamed->ParsedResponse->Result->ExtendType == -1) {
        //Name will be deleted at expiry date
        $return["completed"] = true;
    } else {
        $return["completed"] = false;
    }

    return $return;
}

function BNamed_checkAvailability($params) {
    $result = array();

    $bNamed = new BNamed($params);
    $bNamed->AddParam("command", "check");
    $bNamed->AddParam("domainlist", implode(",", $params["domainNames"]));

    $bNamed->SendRequest();

    if ($bNamed->ParsedResponse->ErrorCode == 0) {
        foreach ($bNamed->ParsedResponse->Result as $namex) {
            foreach ($namex as $DNC) {
                //pa_log("DNC " . print_r($DNC, TRUE), "INFO");
                if ($DNC->AvailabilityCode == 0) {
                    $result[$DNC->SLD . "." . $DNC->TLD] = array('status' => 1, 'msg' => 'This domain is available');
                } else {
                    $result[$DNC->SLD . "." . $DNC->TLD] = array('status' => 2, 'msg' => 'This domain is not available');
                }
            }
        }
    }

//pa_log("Exit checkAvailability with success. Result: " . print_r(json_encode($result), TRUE), "INFO");
    return $result;
}

function BNamed_checkTransfer($params) {
    $result = array();

    $bNamed = new BNamed($params);
    $bNamed->AddParam("command", "check");
    $bNamed->AddParam("domainlist", $params["domainName"]);

    $bNamed->SendRequest();

    if ($bNamed->ParsedResponse->ErrorCode == 0) {
        if ($bNamed->ParsedResponse->Result->Name1->AvailabilityCode == 1) {
            $result = array('status' => 1, 'renewRequired' => 1, 'msg' => "This domain is available for transfer");
        } else {
            $result = array('status' => 2, 'msg' => "This domain is not available for transfer");
        }
    } else {
        $result = array('status' => 2, 'msg' => "Check failed");
    }

    return $result;
}

function BNamed_ValidateAdditionalFields($params) {
    $extFields = array();

    if (strpos("com,net,org,biz,info,be," . $params["noAdditionalFieldsTLDs"], "," . $params["tld"] . ",") === FALSE) {
        $bNamed = new BNamed($params);
        $bNamed->AddParam("command", "TLD");
        $bNamed->AddParam("TLD", $params["tld"]);

        $bNamed->SendRequest();

        if ($bNamed->ParsedResponse->ErrorCode == 0 && $params["operationType"] <= 2) {
            $specialFields = array();

            if ($params["operationType"] == 1) {
                //Registration
                $specialFields = $bNamed->ParsedResponse->Result->TLDs->TLD1->SpecialFields->Registration;
            } elseif ($params["operationType"] == 2) {
                //Transfer
                $specialFields = $bNamed->ParsedResponse->Result->TLDs->TLD1->SpecialFields->TransferTrade;
            }

            $company_or_Indiv = "";
            if (array_key_exists("_Company_or_Indiv", $params["additionalfields"])) {
                $company_or_Indiv = trim($params["additionalfields"]["_Company_or_Indiv"]);
            }

            /*
              $CompString = trim(print_r($specialFields->Company, true));
              $IndString = trim(print_r($specialFields->Individual, true));

              pa_log("MD5 Comp: " . md5($CompString) . " MD5 Indiv: " . md5($IndString), "INFO");

              pa_log("BNamed_ValidateAdditionalFields Company: " . print_r($specialFields->Company, true), "INFO");
              pa_log("BNamed_ValidateAdditionalFields Indiv: " . print_r($specialFields->Individual, true), "INFO");

             * 
             */

            //Do we need to ask different questions to inidividuals or to companies?
            if (($company_or_Indiv == "" || $company_or_Indiv == "null") && print_r($specialFields->Company, true) != print_r($specialFields->Individual, true)) {
                $extFields["_Company_or_Indiv"] = $bNamed->specialFields_Company_or_Indiv("null");
            } else {
                //pa_log("BNamed_ValidateAdditionalFields company_or_Indiv: " . $company_or_Indiv, "INFO");
                if ($company_or_Indiv == "individual") {
                    //pa_log("BNamed_ValidateAdditionalFields Indiv : " . print_r(json_encode($specialFields->Individual), TRUE), "INFO");
                    $extFields = $bNamed->parseSpecialFields($specialFields->Individual, $params["additionalfields"]);
                    if (count($extFields) > 0) {
                        $extFields["_Company_or_Indiv"] = $bNamed->specialFields_Company_or_Indiv("individual");
                    }
                } else {
                    //pa_log("BNamed_ValidateAdditionalFields Company : " . print_r(json_encode($specialFields->Company), TRUE), "INFO");
                    $extFields = $bNamed->parseSpecialFields($specialFields->Company, $params["additionalfields"]);
                    if (count($extFields) > 0) {
                        $extFields["_Company_or_Indiv"] = $bNamed->specialFields_Company_or_Indiv("company");
                    }
                }
            }
        }
    }

    return $extFields;
}

function BNamed_GetEPPCode($params) {
    if ($params['TestMode'] == 'on') {
        $domainName = $params["sld"] . "." . $params["tld"];
        $eppCode = "dummy-code-for-" . $domainName;
        return array('eppcode' => $eppCode, 'completed' => true, 'msg' => _("Domain data has been inquired successfully."), 'failed' => false);
    } else {
        return array('error' => _("This operation cannot be performed for this domain."));
    }
}

function BNamed_inceptDomainFromCppSdk($params) {
    $domainData = json_decode($params['additionalfields']['data'], TRUE);

    // simply propagate existing extData back to Odin Automation
    return array("additionalfields" => $domainData['ext_data']);
}
