<?php

/**
 * Supporting class for Odin-bNamed 1.2
 *
 * @author bim
 */
class BNamed {

    public $BaseUrl = "https://api.bNamed.net/API?";
    public $ConnUrl = '';
    public $RequestResponse = '';
    public $ParsedResponse = '';
    public $params;
    private $switchTechAdmin = ",be,eu,";
    public $instantTransfer = ",be,eu,";
    private $debug = false;
    private $version = "Odin-1.2-20200219";
    
    /**
     * Domain constructor.
     *
            * @param $params
     */
    function __construct($params) {
        $this->CreateConnUrl($params["Username"], $params["Password"]);
        $this->params = $params;
    }

    function CreateConnUrl($uid, $key) {
        $this->ConnUrl = $this->BaseUrl . "version=" . $this->version . "&uid=" . $uid . "&key=" . $key;
    }

    function AddParam($name, $value) {
        $this->ConnUrl .= "&" . $name . "=" . urlencode($value);
    }

    function AddParamsToIdentifyDomain($params) {
        $this->AddParam("sld", $params["sld"]);
        $this->AddParam("tld", $params["tld"]);

        $orderID = $params['additionalfields']['OrderID'];
        if ($orderID > 0) {
            $this->AddParam("orderid", $orderID);
        }
        
        if ($this->debug) {
            $this->AddParam("debug",base64_encode(print_r($params,true)));
        }
    }

    function SendRequest() {
        if (isset($this->params["Proxy"]) && strlen($this->params["Proxy"]) > 0) {
            $aContext = array(
                'http' => array(
                    'proxy' => $this->params["Proxy"],
                    'request_fulluri' => true,
                ),
                'https' => array(
                    'proxy' => $this->params["Proxy"],
                    'request_fulluri' => true,
                )
            );

            $cxContext = stream_context_create($aContext);
            $this->RequestResponse = file_get_contents($this->ConnUrl, False, $cxContext);
        } else {
            $this->RequestResponse = file_get_contents($this->ConnUrl);
        }

        $this->ParsedResponse = simplexml_load_string($this->RequestResponse);
    }

    function ExtractNameservers($params) {
        $nameservers = array();

        for ($i = 1; $i <= 5; $i++) {
            if (isset($params['ns' . $i])) {
                $nameservers[$i] = $params['ns' . $i];
            }
        }

        return implode(",", $nameservers);
    }

    function purchase($params, $PurchaseType) {
        $return = array();

        $bNamed = $this;
        $bNamed->AddParam("command", "purchase");
        $bNamed->AddParam("PurchaseType", $PurchaseType);
        $bNamed->AddParam("sld", $params["sld"]);
        $bNamed->AddParam("tld", $params["tld"]);
        $bNamed->AddParam("NumYears", $params["regperiod"]);
        $bNamed->AddParam("DNSList", $bNamed->ExtractNameservers($params));
        $bNamed->AddParam("DNSType", "E");

        if (strpos($params["autoTrustee"], "," . $params["tld"] . ",") !== FALSE) {
            $bNamed->AddParam("trustee", "1");
        }

        if ($PurchaseType == "B" || $PurchaseType == "T") {
            $bNamed->AddParam("AuthKey", $params["transfersecret"]);
        }

        if (is_array($params["additionalfields"]) || is_object($params["additionalfields"])) {
            foreach ($params["additionalfields"] as $additionalfieldsKey => $additionalfieldsValue) {
                $bNamed->AddParam($additionalfieldsKey, $additionalfieldsValue);
            }
        }

        $bNamed->addContactParams($params);

        $bNamed->SendRequest();

        //Should we ignore 186 $bNamed->ParsedResponse->ErrorCode <> 186
        if ($bNamed->ParsedResponse->ErrorCode > 0) {
            $return["error"] = $bNamed->ParsedResponse->ErrorText . " ( ErrorCode = " . $bNamed->ParsedResponse->ErrorCode . " )";
        } else {
            //ToDo locked status might not be correct
            $return["lockenabled"] = "locked";
            //additionalfields needs to contain at least one item in order for Odin to know this was succesful
            $return["additionalfields"] = array("succes" => "true", "bn-version" => $this->version, "OrderID" => strval(intval($bNamed->ParsedResponse->Result->OrderID)));
        }
        return $return;
    }

    function addContactParams($params) {
        $admin = "Admin";
        $onsite = "Onsite";

        //For historical reasons sometimes Onsite and Admin might need to be switched.
        if (!strpos("," . $this->switchTechAdmin . ",", "," . $params["tld"] . ",") === FALSE) {
            $admin = "Onsite";
            $onsite = "Admin";
        }

        if (isset($params["contactdetails"])) {
            $this->addContactParamsSpecific($params["contactdetails"]["Registrant"], "Licensee", "", true);
            $this->addContactParamsSpecific($params["contactdetails"]["Billing"], "Billing", "", true);
            $this->addContactParamsSpecific($params["contactdetails"]["Admin"], $admin, "", true);
            $this->addContactParamsSpecific($params["contactdetails"]["Tech"], $onsite, "", true);
        } else {
            $this->addContactParamsSpecific($params, "Licensee", "", false);
            $this->addContactParamsSpecific($params, "Billing", "billing", false);
            $this->addContactParamsSpecific($params, $admin, "admin", false);
            $this->addContactParamsSpecific($params, $onsite, "tech", false);
        }
    }

    function addContactParamsSpecific($contactParams, $bNamedPrefix, $odinPrefix, $contactDetails) {
        $bNamed = $this;

        $bNamed->AddParam($bNamedPrefix . "Nr", "-2");

        if ($contactDetails) {
            $bNamed->AddParam($bNamedPrefix . "FirstName", $contactParams["First Name"]);
            $bNamed->AddParam($bNamedPrefix . "Name", $contactParams["Last Name"]);
            $bNamed->AddParam($bNamedPrefix . "Company", $contactParams["company"]);
            $bNamed->AddParam($bNamedPrefix . "EMail", $contactParams["Email"]);
        } else {
            $bNamed->AddParam($bNamedPrefix . "FirstName", $contactParams["firstname"]);
            $bNamed->AddParam($bNamedPrefix . "Name", $contactParams["lastname"]);
            $bNamed->AddParam($bNamedPrefix . "Company", $contactParams["companyname"]);
            $bNamed->AddParam($bNamedPrefix . "EMail", $contactParams["email"]);
        }

        $bNamed->AddParam($bNamedPrefix . "Street", $contactParams["address1"]);
        $bNamed->AddParam($bNamedPrefix . "PostCode", $contactParams["postcode"]);
        $bNamed->AddParam($bNamedPrefix . "Residence", $contactParams["city"]);
        $bNamed->AddParam($bNamedPrefix . "Country", $contactParams["country"]);
        $bNamed->AddParam($bNamedPrefix . "Phone", $contactParams["phonenumber"]);
    }

    function specialFields_Company_or_Indiv($defaultValue) {
        $status = ($defaultValue == "null" ? 2 : 1);
        return array("type" => "dropdown",
            "title" => "Company or individual",
            "status" => $status,
            "options" => "null|Select owner type,company|Owner is a company,individual|Owner is an individual",
            "defaultValue" => $defaultValue,
            "required" => 1);
    }

    function parseSpecialFields($fields, $additionalfields) {
        $returnFields = array();

        foreach ($fields->field as $field) {
            $include = true;
            $returnField = array();
            $attributes = $field->attributes();
            $descriptionArray = split(":", $field->Description_English);

            $name = $attributes["name"] . "";
            //ToDo: give client option to activate trustee
            if ($name != "trustee" && $name != "eppKey") {
                if ($field->Type == "Select") {
                    $returnField["type"] = "dropdown";
                    $returnField["options"] = "";
                    foreach ($field->Options->Option as $option) {
                        if (strlen($returnField["options"]) > 0) {
                            $returnField["options"] .= ",";
                        }
                        $returnField["options"] .= $option->Value . "|" . $option->Description_English;
                    }
                } else {
                    $returnField["type"] = "text";
                }

                $returnField["title"] = $descriptionArray[0];
                if (count($descriptionArray) > 1) {
                    $returnField["description"] = $descriptionArray[1] . ". ";
                }

                if ($attributes["localAddress"] == "true" || $attributes["localAddress"] == "false") {
                    $hasLocalAddress = "";
                    if (array_key_exists("hasLocalAddress", $additionalfields)) {
                        $hasLocalAddress = $additionalfields["hasLocalAddress"];
                    }

                    $returnFields["hasLocalAddress"] = array("type" => "dropdown",
                        "title" => "Owner has local address",
                        "status" => 1,
                        "options" => "null|Select if owner has local address,true|The owner does have a local address,false|The owner doesn't have a local address",
                        "defaultValue" => $hasLocalAddress,
                        "required" => 1);

                    if ($hasLocalAddress == "true" && $attributes["localAddress"] == "true") {
                        //okay
                    } elseif ($hasLocalAddress == "false" && $attributes["localAddress"] == "false") {
                        //okay
                    } else {
                        $include = false;
                    }
                }

                if ($attributes["required"] == "true") {
                    $returnField["required"] = 1;
                } else {
                    $returnField["required"] = 0;
                }

                if (array_key_exists($name, $additionalfields)) {
                    if (strlen($additionalfields[$name]) > 0) {
                        //Okay
                        $returnField["status"] = 1;
                    } elseif ($returnField["required"] == 0) {
                        $include = false;
                    } else {
                        //Fail
                        $returnField["status"] = 2;
                    }
                } else {
                    //Needed
                    $returnField["status"] = 3;
                }

                if ($include) {
                    $returnFields[$name] = $returnField;
                }
            }
        }
        //pa_log("parseSpecialFields: " . print_r(json_encode($returnFields), TRUE), "INFO");
        return $returnFields;
    }

}
